/*
 * AFA fragmentation and reassembly library header file
 *
 * Jeremy Elson, USC/ISI
 *
 * For details on the theory of AFA fragmentation please see
 * http://www.circlemud.org/~jelson/writings/afa
 *
 * $Id: afa_fragmentation.h,v 1.5 2000/07/11 07:56:07 jelson Exp $
 */

/*************************** Public Interface *****************************/

/* downcalls provided by the fragmentation library */
void fragmentation_init(void);
void fragmentation_shutdown(void);
void process_incoming_fragment(caddr_t message, int length,
                               struct timeval packet_time);
void fragment_message(caddr_t message, int length);

/* upcalls needed by the fragmentation library */
void send_fragment_upcall(caddr_t message, int length);
void deliver_packet_upcall(caddr_t message, int length,
                           struct timeval packet_time);

/* debugging */
char *asc2hex(caddr_t buffer, int length);

#define FRAG_DATA_LEN        23        /* data size in fragments */
#define MAX_PACKETS          128       /* 7 bits of frag ID */
#define MAX_PACKET_SIZE      8196




/**************** Private Definitions (not part of API) *********************/

#ifdef __KERNEL__
#include <linux/time.h>
#else
#include <sys/time.h>
#endif

#if defined(__AFA_FRAGMENTATION_C__) || defined(__RPC_KM_C__)

u_int32_t calc_checksum(caddr_t message, int length);

#define FRAG_INTRO 0
#define FRAG_DATA  1

/* header that RPC daemon sends before a fragment: the part common to
 * both fragment intro and fragment data */
typedef struct {
  u_int packet_type:1;
  u_int frag_id:7;
  u_int16_t byte_id;
} frag_common_header_t;

/* header that RPC daemon sends before a fragment: for a "fragment
 * info" type packet.  */
typedef struct {
  frag_common_header_t hdr;
  u_int32_t checksum;
} frag_intro_t;

/* header that RPC daemon sends before a fragment: for a "fragment
 * data" type packet.  */
typedef struct {
  frag_common_header_t hdr;
  
  char data[FRAG_DATA_LEN];
} frag_data_t;


typedef struct {
  u_int32_t checksum;
  u_int16_t length;
  caddr_t data;
  struct timeval time;
} rpc_packet_t;


#endif
