/*
 * Radiometrix RPC driver kernel module header file
 *
 * Jeremy Elson, USC/ISI
 *
 * $Id: krpc.h,v 1.17 2001/04/04 21:44:23 jelson Exp $
 */
/****************** Public Interface ***************************************/

#define RPC_MAJOR_NUMBER_DEFAULT  240

/*
 *     Minor number 0: Raw RPC interface (just like the current driver)
 *     Minor number 1:  Fragmentation interface so that packets larger
 *                     than 27 bytes can be transmitted and/or
 *                     received.
 *     Minor number 2: Packets are delivered with the "rpc_augmented"
 *                     structure prepended (i.e., before the data).
 *                     This structure contains data such as the time of
 *                     reception of the  packet.
 *     Minor number 3: Both 1 and 2
 */
#define RPC_MODE_COOKED     1
#define RPC_MODE_AUGMENTED  2
#define RPC_MODE_NETIF      4

/* When using the file interface (in any minor mode), requesting a
 * read with the RPC_MSG_PEEK bit "or"d into the length will return
 * the first message in the queue without actually dequeueing the
 * message.  A subsequent read call will return the same data.  (This
 * feature is similar to MSG_PEEK in recv(2).)
 *
 * In other words, the call:
 *       read(fd, buf, 10 | RPC_MSG_PEEK);
 * Will request a 10 byte read without removing the message from the queue.
 */
#define RPC_MSG_PEEK (1 << 30) /* using 30 instead of 31 to avoid playing
				  with signed-ness issuses */


#ifdef __KERNEL__
#include <linux/time.h>
#else
#include <sys/time.h>
#endif

/* the "augmented data" structure */
struct rpc_augmented {
  struct timeval packet_time;
  int spare1;  /* spares can be renamed without requiring userspace recompile*/
  int spare2;
  int spare3;
};


/****************** End of Public Interface ********************************/


/* debugging support */
void rdebug_real(char *fmt, ...)
  __attribute__ ((format (printf, 1, 2)));

#define RDEBUG(message_level, args...) do { \
   if (krpc_debug_level >= message_level) rdebug_real(args); \
} while(0)

#ifndef __RPC_KM_C__
extern int krpc_debug_level;
#endif


#ifdef __RPC_KM_C__ /* defines only needed in rpc-km.c and friends */

#define RPC_POLL_INTERVAL 1
#define RPC_BUFFER_SIZE 32

#define MAX_RPC_PROC_ENTRIES 10
#define KRPC_PROC_BASENAME "krpc"

#define RPC_RETRY_INTERVAL (HZ * 2) /* how often we try to reset after error */
#define RPC_TIMEOUT_TIME (HZ/2)     /* how long we are willing to wait after
                                       a TX request before declaring radio
                                       dead */

#define CA_HOLDOFF_DEFAULT  (HZ/20)
#define CA_HOLDOFF_RAND_DEFAULT (HZ/20)

/* possible reasons why we are running rpc_run_radio */
#define RPC_TIMER     1
#define RPC_INTERRUPT 2
#define RPC_WRITE     3

#define run_radio(reason) { \
  rpc_queue.data = (void *) reason; \
  rpc_queue.routine = (void *) (void *) rpc_run_radio;\
  queue_task(&rpc_queue, &tq_immediate); \
  mark_bh(IMMEDIATE_BH); }

#define UNKNOWN_TIME(t) (t.tv_sec == unknown_time.tv_sec &&  \
                         t.tv_usec == unknown_time.tv_usec)

static void rpc_run_radio(void *data);
void send_fragment_upcall(caddr_t message, int legnth);
void enqueue_packet(caddr_t message, int length, struct timeval packet_time);


#endif /* __RPC_KM_C__ */
