/*
 * This is the API for librpc, the user-land version of the RPC library.
 *
 * This include file is meant to be used by applications that talk
 * directly to the radio using the user-land "librpc" library.  Note
 * that only one such application can run at a time, because the
 * application will be managing the physical device (actually flipping
 * pins on the parallel port).
 *
 * This is a fairly low-level interface to the radio, and as such
 * exposes the radio's limitation of 27 byte packets.
 *
 * J Elson 2 March 2000
 *
 * $Id: rpc.h,v 1.2 2000/04/09 21:23:25 jelson Exp $
 */

/* Public Functions - for users of the user library */

/* Initialize the library, for now assumes port=LPT1.  Won't block. */
int rpc_init();

/* Reset the RPC.  Called by rpc_init() but can be called separately. */
void rpc_reset();

/* send_message sends a message.  This might result in an incoming
 * message being received as well.  Messages can not be more than 27
 * bytes.  Returns the number of characters sent, or -1 if an error
 * occurred. */
int rpc_send_message(caddr_t buf, int length);

/* This function, when called, polls the radiometrix for incoming
 * messages.  If one exists, the incoming message callback is called
 * (see below), and it returns 1.  If no message is waiting, it
 * returns 0. */
int rpc_receive_message();

/* register_incoming_message_callback() lets the user pass a function
 * pointer to a function that should be called when messages are
 * received */
typedef void (*incoming_message_callback_prototype)(caddr_t message, int length);
void rpc_register_incoming_message_callback(incoming_message_callback_prototype);

/* register_termination_callback lets the user pass a function pointer
 * to a function that should be called when the library terminates, if
 * control-c is hit in the middle of send_message or receive_message.
 * (The library traps control-c) */
typedef void (*termination_callback_prototype)();
void rpc_register_termination_callback(termination_callback_prototype);

/* Send a control message -- write value to address */
void rpc_send_control(char address, char value);

/************* Private functions - not part of the interface ************/
/* (these really should be in a different include file altogether)      */

#ifdef __LIBRPC_C__

static void terminate();
static void sighandler();

#define TIMEOUT(func, time) do { \
   alarm(time); \
   signal(SIGALRM, alarmhandler); \
   func; \
   alarm(0); \
} while(0)

#endif

