/*
 * This is a private header file used for the implementation of RPCD,
 * the RPC Daemon.  It is not a public interface definition and should
 * not be used by applications.
 *
 * J. Elson 2 March 2000
 *
 * $Id: rpcd.h,v 1.5 2001/04/04 21:42:34 jelson Exp $
 */
/****************************************************************************/
/*********** Private Definitions Below- not for public use ******************/
/****************************************************************************/
/* implementation begins */

#include <sys/uio.h>    /* needed for iovec */
#include <sys/socket.h> /* needed for sockaddr_in */


#define RPCLIB_INTERNAL_PORT 5760      /* 17 Tishrei */
#define RPC_FRAG_DEBUG_PORT  5770
#define SELECT_TIME          1000      /* time in usec between polls */
#define SENDER_IPG           20000     /* interpacket gap on sender */
#define DEBUG_BUF_SIZE       1024      /* size of debugging buffer */

/* possible modes that the radiometrix library can be in */
enum radiometrix_mode_t { NORMAL, DEBUG, FRAG_DEBUG };

/* server_setup port types */
enum port_type_t { TCP, UDP };

/* structure used by daemon to keep track of all connected clients */
typedef struct {
  int fd;
  int dead;
} client_t;

/* header sent between clients and the RPC daemon before data that
 * they wish the daemon to send. */
typedef struct {
  u_int32_t length;
} internal_comm_hdr;



/****************************************************************************/
/********************** Internal Function Prototypes ************************/
/****************************************************************************/


/* util.c */
void rinit_debug(char *name);
void rdebug_real(char *fmt, ...)
  __attribute__ ((format (printf, 1, 2)));
void rdebug_print_message(int level, char *prefix, caddr_t message,int length);
int nonblock(int fd);
int tcp_open(caddr_t host, char *service, int port);
int tcp_open_retry(char *site, int port);
int init_server_connection(char *hostname, u_int16_t port);
int stream_read(int fd, caddr_t ptr, size_t nbytes);
int stream_write(int fd, caddr_t ptr, size_t nbytes);
int multi_stream_write(int fd, struct iovec *iov, int iovcnt);
int create_udp_source(char *hostname, u_int16_t dest_port,
                      struct sockaddr_in *sa);
void *check_malloc(size_t size);
void process_incoming_fragment_wrapper(caddr_t message, int length);


/* daemon.c */
void run_daemon(void);
#ifdef __DAEMON_C__
static void accept_new_clients();
static void purge_client_array();
static int  check_client_messages(fd_set *read_set);
void check_incoming_fragments();
static void process_outgoing_message(int fd_index);
#endif

/* realradio.c */
int connect_to_daemon(void);
int send_to_server(caddr_t message, int length);
int send_to_daemon(caddr_t message, int length);
int server_setup(int server_port, int port_type);


/* Debugging support */

#ifndef __UTIL_C__
extern int radiometrix_debug_level;
#endif

char *asc2hex(caddr_t buffer, int length);

#define MALLOC(type, num)  (type *) check_malloc((num) * sizeof(type))

#define RDEBUG(message_level, args...) do { \
   if (radiometrix_debug_level >= message_level) rdebug_real(args); \
} while(0)

#define RAW_RDEBUG(x, y) if (radiometrix_debug_level >= (x)) do { \
  y; \
} while (0)


