/*
 * lossrate.c
 * 
 * uses scadds_addr as an address
 *
 * $Id: klossrate.c,v 1.3 2000/05/12 17:00:10 girod Exp $
 */

#define MAX_MSG_SIZE   1024

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <string.h>
#include <assert.h>
#include <signal.h>


#define MAGIC 123456
#define NUMSIZES 1     // set the number of distinct packet sizes here
#define TRACEHIST 128
#define MAXSENDERS 5   // set the maximum number of senders here

u_int32_t pSizes[NUMSIZES] = {100}; 
// configure the expected set of distinct packet sizes 
// for example, {23, 47, 95, 191, 383};

char *rcsid = "$Id: klossrate.c,v 1.3 2000/05/12 17:00:10 girod Exp $";

struct pkt {
  u_int32_t src;
  u_int32_t seq;
  u_int32_t magic;
  u_int8_t interval;
  u_int8_t ttl;
  char connect[MAXSENDERS][MAXSENDERS];
  char padding[1024];
};

int radio_fd = -1;
u_int32_t radio_addr;
u_int32_t sendSeq;
struct pkt outgoing;

struct timeval wakeup;

char globalInuse[MAXSENDERS];

struct sender {
  u_int32_t lastSeq;
  u_int8_t matrix[NUMSIZES][TRACEHIST];
  u_int32_t loss;
  u_int32_t last;
  int isValid;
  int valid;
};

struct sender senders[MAXSENDERS];

int nosend;
int secs;
int sendInterval;


void init()
{
  /* sendInterval is a ttl ... */
  sendInterval = 5;
  sendSeq = 1;
  memset(&(senders), 0, sizeof(senders));
  memset(&(outgoing), 0, sizeof(outgoing));
}



/* read one packet off the wire and print it out */
void read_packet()
{
  char buf[MAX_MSG_SIZE];
  int bytes_read;
  int diff;

  bytes_read = read(radio_fd, buf, MAX_MSG_SIZE);

  if (bytes_read < 0) {
    perror("app: error reading from network");
    /*    exit(1); */
  } else if (bytes_read < pSizes[0]) {
    fprintf(stderr, "app: packet too small\n");
    /*     exit(1); */
  } else {
    struct pkt *p = (struct pkt *)&(buf[0]);
    if (p->magic != MAGIC) {
      fprintf(stderr, "app: wrong protocol\n");
    }
    else {
      int i;
      int sender = p->src;
      struct sender *s = senders+sender;
      
      if (sender > MAXSENDERS) {
	fprintf(stderr, "app: src address %d too big\n", sender);
	return;
      }

      for (i=0; i<NUMSIZES; i++) {
	if (bytes_read == pSizes[i]) {

	  if (bytes_read > (sizeof(struct pkt) - sizeof(p->padding))) {
	    int x,y;
	    for (x=0; x<MAXSENDERS; x++) 
	      for (y=0; y<MAXSENDERS; y++) 
		if (p->connect[x][y] && (y != radio_addr))
		  outgoing.connect[x][y]=p->connect[x][y];
	  }

	  // Reset our seconds argument to what we heard from the net.
	  if (p->interval)
	    secs = p->interval;

	  // If the ttl on that new seconds value is bigger than our
	  // current ttl then we reset our ttl to that value.
	  // Note that we'll decrement before we send again so there
	  // is no possibility of looping.
	  if (p->ttl > sendInterval && !sendInterval)
	    sendInterval = p->ttl;
	  
	  /* figure out when to send next. */
	  if (p->src < radio_addr)
	    diff = radio_addr - p->src;
	  else
	    diff = MAXSENDERS - p->src + radio_addr;

	  diff = diff*secs;
	  wakeup.tv_sec = diff/10;
	  wakeup.tv_usec = (diff%10)*100000;
  
	  if (!s->lastSeq || (s->lastSeq > p->seq)) {
	    s->lastSeq = p->seq;
	    s->valid = s->lastSeq;
	    s->isValid = 0;
	  }
	  
	  /* clear lost packets in matrix */
	  while (s->lastSeq < p->seq) {
	    int j;
	    s->lastSeq++;
	    if (!s->isValid && (s->valid%TRACEHIST == s->lastSeq%TRACEHIST))
	      s->isValid = 1;
	    for (j=0; j<NUMSIZES; j++) {
	      s->matrix[j][s->lastSeq%TRACEHIST]=0;
	    }
	  }
	  
	  /* set this packet stat */
	  s->matrix[i][s->lastSeq%TRACEHIST] = 1;
	  break;
	}
      }
    }
  }
}


/* Send a message using the radiometrix kernel module, and re-set the
 * alarm so we send another packet one second from now. */
void send_packets(int signal)
{
  int i;

  printf("Sending %d...\n", sendSeq);

  outgoing.magic = MAGIC;
  outgoing.seq=sendSeq;
  outgoing.src=radio_addr;
  if (--sendInterval)
    outgoing.interval = secs;
  else
    outgoing.interval = 0;
  
  for (i=0; i<NUMSIZES; i++) {
    write(radio_fd, (char *)&outgoing, pSizes[i]);
  }
  
  sendSeq++;

  /* print a status report for each sender */
  for (i=0; i<MAXSENDERS; i++) {
    int cap = 1;
    int index = 0;
    int sum[NUMSIZES];
    int j;
    struct sender *s = senders+i;

    /* is it in use..? */
    if (!s->lastSeq) continue;
    
    memset(&(sum[0]), 0, sizeof(sum));
    
    if ((s->last) && (s->last == s->lastSeq)) {
      s->loss++;
      printf("#%d ??(%3d)??", i, s->loss);
    }
    else {
      s->loss = 0;
      s->last = s->lastSeq;
      printf("#%d  >>>>>>>>", i);
    }
    
    index = 0;
    for (j=0; j<NUMSIZES; j++) {
      printf(" %3d bytes ",pSizes[j]);
    }
    printf("\n");
    while (cap <= TRACEHIST) {
      if (!s->isValid && (cap > (s->lastSeq - s->valid))) break;
      
      if (index >= s->loss) 
	for (j=0; j<NUMSIZES; j++) {
	  if (s->matrix[j][(TRACEHIST+s->lastSeq+s->loss-index)%TRACEHIST])
	    sum[j]++;
	}
      
      index++;
      
      if (index >= cap) {
	printf(" Last %3d:  ", cap);
	for (j=0; j<NUMSIZES; j++) 
	  printf("  %6.2f%%  ", ((float)sum[j]*100)/((float)cap));
	printf("\n");

	if (cap==16) {
	  outgoing.connect[i][radio_addr] = 
	    (int)((((float)sum[3]*100)/16.0)*0.75 +
		  (((float)sum[3]*100)/16.0)*0.25);
	}

	cap*=2;
      }
    }
  }

  printf("\nConnectivity matrix:");
  {
    int x,y;
    
    memset(&globalInuse, 0, sizeof(globalInuse));
    
    for (y=0; y<MAXSENDERS; y++)
      for (x=0; x<MAXSENDERS; x++)
	if (outgoing.connect[x][y]) 
	  globalInuse[x] = globalInuse[y] = 1;
      
    for (y=-1; y<MAXSENDERS; y++) 
      if (y<0 || globalInuse[y])
	for (x=-1; x<MAXSENDERS; x++)
	  if (x<0 || globalInuse[x]) {
	    if (x==y) {
	      if (x<0)
		printf("\nS\\D ");
	      else
		printf(" -- ");
	      continue;
	    }

	    if (y<0) 
	      printf(" #%d ",x); 
	    else {
	      if (x<0)
		printf("\n#%d  ",y);
	      else
		printf("%3d ",outgoing.connect[x][y]);
	    }
	  }
    printf("\n\n");
  
    x = MAXSENDERS*secs;
    wakeup.tv_sec = x/10;
    wakeup.tv_usec = (x%10)*100000;
  }
}


int main(int argc, char *argv[])
{

  /* Make sure we have the arguments we need */
  if (argc < 3) {
    fprintf(stderr, 
	    "Usage: %s <address> <tenths (neg for no send)>\n\n"
	    "Some configuration must be done in the source code:\n"
	    "* The packet sizes to send in each test: array pSizes.\n"
	    "* MAXSENDERS is the total number of devices in the network.\n\n"
	    "Note that connectivity matrix data comes from the 'last 16' average.\n"
	    "The tenths (of a second) argument configures the sender spacing.  \n"
	    "The senders space themselves out according to the tenths value.\n"
	    "The address argument is and index from 1 to %d.\n",
	    argv[0], MAXSENDERS);
    exit(1);
  }

  radio_addr = atoi(argv[1])-1;
  if (radio_addr >= MAXSENDERS) {
    fprintf(stderr, "%s: Illegal address value.\n", argv[0]);
    exit(1);
  }
  nosend = 0;
  secs = atoi(argv[2]);
  if (secs<0) {
    nosend=1;
    secs = -secs;
  }

  init();

  /* open the radiometrix device driver */
  if ((radio_fd = open("/dev/rpcc", O_RDWR)) < 0) {
    perror("open of /dev/rpcc failed");
    exit(1);
  }

  /* Send a ping every 1/persecond seconds */
  //  signal(SIGALRM, send_packets);
  //ualarm(micros, micros);
  //alarm(secs);

  /* Now, loop forever: check for incoming packets, time out, send our
   * own packet.  First set up the signal handler. */
  wakeup.tv_sec=0;
  wakeup.tv_usec=0;

  do {
    fd_set fds;
    FD_ZERO(&fds);
    FD_SET(radio_fd, &fds);
    if (0 == select(radio_fd+1, &fds, NULL, NULL, &wakeup))
      send_packets(1);
    else
      read_packet();
  } while (1);


  fprintf(stderr, "app: exiting[%d]\n", getpid());
}
