#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>


int fd;
int timer_expired;

void alarm_handler()
{
  timer_expired = 1;
}


int nonblock(int on)
{
  int val;

  if ((val = fcntl(fd, F_GETFL, 0)) < 0)
    return val;

  if (on)
    val |= O_NONBLOCK;
  else
    val &= ~(O_NONBLOCK);

  if (fcntl(fd, F_SETFL, val) < 0)
    return val;

  return 0;
}


double get_loss()
{
  char buf[128];
  int seqno_start, seqno_end, count;
  int retval;
  int stage = 0;
  int final_packet_received = 0;

  /* first clear the queue */
  nonblock(1);
  errno = 0;
  while (read(fd, buf, 128) > 0)
    ;
  if (errno != EAGAIN) {
    perror("error trying to clear rpc queue");
    exit(1);
  }


  /* read the first packet - waiting a max of 2 sec */
  nonblock(1);
  retval = -1;
  timer_expired = 0;
  signal(SIGALRM, alarm_handler);
  alarm(2);

  while (retval < 0 && !timer_expired) {
    retval = read(fd, buf, 128);
    if (retval < 0 && errno != EAGAIN) {
      perror("reading from radio");
      exit(1);
    }
    if (retval > 0) {
      buf[retval] = '\0';
      printf("%s", buf);
    }
  }
  alarm(0);

  /* if we didn't get any packets in 2 seconds, call it 0% success */
  if (timer_expired)
    return 0.0;

  if (!sscanf(buf, "%d", &seqno_start)) {
    fprintf(stderr, "protocol error?  got line: '%s'\n", buf);
    exit(1);
  }
  count = 0; /* we _don't_ count the first packet */

  stage = 0;
  timer_expired = 0;
  final_packet_received = 0;
  signal(SIGALRM, alarm_handler);
  alarm(2);
  nonblock(1);

  for (;;) {

    if (timer_expired) {
      if (stage == 1) {
        break;
      }
      if (stage == 0) {
        stage = 1;
        timer_expired = 0;
        signal(SIGALRM, alarm_handler);
        alarm(2);
        fprintf(stderr, "...entering final stage\n");
      }
    }

    if ((retval = read(fd, buf, 128)) < 0) {
      if (errno == EAGAIN)
        continue;
      perror("read");
      exit(1);
    }

    /* packet was received */
    buf[retval] = '\0';
    printf("%s", buf);

    sscanf(buf, "%d", &seqno_end);
    count++;

    if (stage == 1) {
      final_packet_received = 1;
      fprintf(stderr, "....that was the final packet, seqno %d\n", seqno_end);
      break;
    }
  }

  if (!final_packet_received)
    return 0.01;

  /* figure out how many transmissions were required for 100 receptions */
  return 100 * (count / (double) (seqno_end - seqno_start));
}


int main(int argc, char *argv[])
{
  FILE *f;
  int i;
  double loss, dist, fudge;
  char input[2000];

  if ((fd = open("/dev/rpc", O_RDWR)) < 0) {
    perror("/dev/rpc");
    exit(1);
  }

  if (argc != 2) {
    fprintf(stderr, "usage: %s <output-filename>\n", argv[0]);
    exit(1);
  }

  if (!(f = fopen(argv[1], "a"))) {
    perror(argv[1]);
    exit(1);
  }

  fprintf(f, "# %s\n", argv[1]);
  fprintf(f, "# Distance    Successrate\n");

  while (1) {
    fprintf(stderr, "\n\nEnter distance: ");
    fflush(stderr);

    while (fgets(input, 1024, stdin) == NULL || (dist = atof(input)) < 0.01) {
      fprintf(stderr, "invalid, try again: ");
    }

    if (!strcmp(input, "quit\n")) {
      fclose(f);
      exit(1);
    }

    dist = atof(input);

    fprintf(stderr, "testing with dist %f...\n", dist);

    for (i = 0; i < 20; i++) {
      loss = get_loss();
      printf("Success Rate: %.2f%%\n", loss);
      fudge = (rand() % 200) / 1000.0;
      fprintf(f, "%f %f\n", dist + fudge, loss);
      fflush(f);
    }
  }

}

