/*
 * rpctorture.c: A torture test and performance meter for the RPC
 * module.  It simply transmits data as fast as it can, and calculates
 * how quickly the data was sent in the end.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

static int count;
static struct timeval start;

/*
 * code to return the time difference between a and b (a-b).
 * always returns a nonnegative value (floors at 0).
 */
void timediff(struct timeval *rslt, struct timeval *a, struct timeval *b)
{
  if (a->tv_sec < b->tv_sec)
    rslt->tv_usec = rslt->tv_sec = 0;
  else if (a->tv_sec == b->tv_sec) {
    if (a->tv_usec < b->tv_usec)
      rslt->tv_sec = rslt->tv_usec = 0;
    else {
      rslt->tv_sec = 0;
      rslt->tv_usec = a->tv_usec - b->tv_usec;
    }
  } else {                      /* a->tv_sec > b->tv_sec */
    rslt->tv_sec = a->tv_sec - b->tv_sec;
    if (a->tv_usec < b->tv_usec) {
      rslt->tv_usec = a->tv_usec + 1000000 - b->tv_usec;
      rslt->tv_sec--;
    } else
      rslt->tv_usec = a->tv_usec - b->tv_usec;
  }
}


void print_results()
{
  double t;
  struct timeval stop, elapsed;

  gettimeofday(&stop, NULL);
  timediff(&elapsed, &stop, &start);
  t = elapsed.tv_sec + (elapsed.tv_usec / 1000000.0);
  putchar('\n');
  printf("%d packets (%d bytes) transmitted in %.3f seconds =\n",
         count, 27 * count, t);
  printf("%.2f packets/sec; %.2f bytes/sec; %.2f bits/sec\n",
         count / t, (27.0*count)/t, (27.0*8*count)/t);
  exit(1);
}


#define MSGLEN 27

int main(int argc, char *argv[])
{
  char buf[1024];
  int i, fd;

  if ((fd = open("/dev/rpc", O_RDWR)) < 0) {
    perror("/dev/rpc");
    exit(1);
  }

  signal(SIGINT, print_results);

  /* record the time the test started */
  printf("Transmitting...\n");
  srand(time(0));
  gettimeofday(&start, NULL);
  count = 0;

  /* send randomly generated 27-byte packets forever (until user hits
   * ctrl-c */
  for (;;) {

    sprintf(buf, "%010d ", count);
    for (i = 11; i < MSGLEN; i++)
      buf[i] = (rand() % 26) + 'a';
    buf[MSGLEN-1] = '\n';
    buf[MSGLEN] = '\0';

    if (write(fd, buf, MSGLEN) < 0) {
      perror("write failed");
      exit(1);
    }

    count++;
    if (!(count % 50))
      putchar('.');
    fflush(stdout);

  }
}


