#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>

/* This function makes a file descriptor non-blocking */
int nonblock(int fd)
{
  int val;

  if ((val = fcntl(fd, F_GETFL, 0)) < 0)
    return val;
  val |= O_NONBLOCK;
  if (fcntl(fd, F_SETFL, val) < 0)
    return val;

  return 0;
}



int main(int argc, char *argv[])
{
  FILE *f;
  int fd, i, retval;
  fd_set s;
  char buf[4096];

  if (argc < 2) {
    fprintf(stderr, "usage: %s <file-to-read-using-select>\n", argv[0]);
    exit(1);
  }

  if ((f = fopen(argv[1], "r")) == NULL) {
    perror(argv[1]);
    exit(1);
  }

  fd = fileno(f);

  if (nonblock(fd) < 0) {
    perror("Oops: setting fd to nonblocking mode had a problem");
    exit(1);
  }

  for (;;) {
    FD_ZERO(&s);
    FD_SET(fd, &s);
    printf("Going to sleep using select...\n");
    retval = select(fd+1, &s, NULL, NULL, NULL);
    printf("Select has awakened!  Joy!\n");

    if (retval < 0) {
      perror("...but select had an error");
      exit(1);
    }

    if (retval == 0 || !FD_ISSET(fd, &s)) {
      printf("...but the descriptor is not readable.  Sigh.\n");
      exit(1);
    }

    retval = read(fd, buf, 4096);

    if (retval < 0) {
      perror("...but trying to read gave us an error");
      exit(1);
    }

    if (retval == 0) {
      printf("...but the retval of read was 0 for some odd reason!\n");
      exit(1);
    }

    printf("And we successfully got data!  %d bytes worth!  Here it is...\n",
           retval);
    buf[retval] = '\0';

    for (i = 0; i < retval; i++)
      if (!isprint(buf[i]) && !isspace(buf[i]))
          buf[i] = '.';

    printf("%s\n", buf);
  }
}
