
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/uio.h>
#include <signal.h>

#include "rpcd_api.h"
#include "rpcd.h"

int daemon_fd = -1;


void pipe_received(int signal)
{
  RDEBUG(4, "connection to daemon lost");
  daemon_fd = -1;
}


  
/*
 * Try to establish a connection to the radiometrix control daemon.  On
 * success, daemon_fd is set to the FD of the tcp socket, and the
 * function returns 0.  On failure it returns -1.
 */
int connect_to_daemon()
{
  signal(SIGPIPE, pipe_received);
  RDEBUG(5, "connect_to_daemon: trying to connect to daemon");
  daemon_fd = tcp_open_retry("localhost", RPCLIB_INTERNAL_PORT);

  if (daemon_fd < 0) {
    RDEBUG(5, "connect_to_daemon: failed");
    return -1;
  } else {
    RDEBUG(5, "connect_to_daemon: success");
    return 0;
  }
}



/* This a function run by the library which is part of client code
 * (NOT the daemon) when the client wants to send a packet.  The
 * client sends the packet to the daemon, where the daemon will read
 * it out of the socket and send it to the physical radio. */

#define MAX_RETRIES 5

int send_to_daemon(caddr_t message, int length)
{
  internal_comm_hdr hdr;
  struct iovec iov[2];
  int retval, retries;

  /* if our daemon fd is invalid, try reopening a connection to the
   * server.  if that fails, return an error. */
  if (daemon_fd < 0) {
    if (connect_to_daemon() < 0 || daemon_fd < 0)
      return -1;
  }

  /* make sure the message is sane */
  if (length <= 0 || message == NULL)
    return -1;

  hdr.length = length;

  /* write our header followed by the data.  if daemon is dead, reconnect. */
  retries = 0;
  retval = -1;
  iov[0].iov_base = (caddr_t) &hdr;
  iov[0].iov_len = sizeof(hdr);
  iov[1].iov_base = (caddr_t) message;
  iov[1].iov_len = length;

  while (retval < 0 && retries <= MAX_RETRIES) {
    retries++;
    RDEBUG(5, "send_to_daemon: calling multi_stream_write");
    if ((retval = multi_stream_write(daemon_fd, iov, 2)) < 0) {
      close(daemon_fd);
      daemon_fd = -1;
      if (connect_to_daemon() < 0)
        return -1;
    }
  }

  return retval;
}
