/*
 * Radiometrix High-Level API.  This file just contains functions that
 * implement the public interface.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rpcd_api.h"
#include "rpcd.h"


/*************************************************************************/


int radiometrix_mode;
char *debug_hostname;


/*
 * Set the debug level.  Can be called before init (as
 * counterintuitive as this sounds); and, in fact, should be called
 * before the init if you wish to see init-related debug messages.
 */
void radiometrix_set_debug_level(int level)
{
  radiometrix_debug_level = level;
}


/*
 * Initialization into the "normal" mode -- i.e. running on real
 * radiometrix hardware.
 */
int radiometrix_init()
{
  int error = 0;

  radiometrix_mode = NORMAL;

  rinit_debug("radiometrix library");
  error += connect_to_daemon();
  /* other initialization if needed... */

  return error;
}


/*
 * Initialization into debug mode.  Sends unfragmented packets to the server.
 */
int radiometrix_init_debug(char *hostname, u_int16_t port)
{
  int error = 0;

  radiometrix_mode = DEBUG;

  rinit_debug("radiometrix library");
  error += init_server_connection(hostname, port);
  /* other initialization if needed... */

  return error;
}


#if 0

/*
 * Initialization into "fragment debug" mode.  Goes through normal
 * fragmentation but sends fragments to the server instead of the
 * radio.
 */
int radiometrix_init_frag_debug(char *hostname)
{
  int error = 0;

  radiometrix_mode = FRAG_DEBUG;

  rinit_debug("radiometrix library");

  debug_hostname = strdup(hostname);
  error += connect_to_daemon();               /* daemon to send messages */

  return error;
}
#endif


/*
 * Send a message using the radiometrix library.
 */
int radiometrix_send(caddr_t message, int length)
{
  /* print the message to the console, if debug level is high enough */
  rdebug_print_message(10, "sending message: ", message, length);

  /* dispatch the message appropriately depending on our mode */
  switch (radiometrix_mode) {
  case NORMAL:
  case FRAG_DEBUG:
    return send_to_daemon(message, length);
    break;
  case DEBUG:
    return send_to_server(message, length);
    break;
  default:
    RDEBUG(1, "radiometrix in unknown mode!");
    return -1;
    break;
  }
}

