/*
 * Apitest: Sample use of radiometrix api.  Simply sends a packet to
 * the server once every second, and, every second, prints any packets
 * received.  Takes two arguments: first, the name of the debugging
 * server to use; second, the contents of the message.
 */

#define MAX_MSG_SIZE   512

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <string.h>
#include <assert.h>
#include <signal.h>

#include "../radiometrix.h"


char *outgoing_message;

/* Send a message using the radiometrix library, and re-set the alarm
 * so we send another packet one second from now. */
void send_packet(int signal)
{
  alarm(10);
  fprintf(stderr, "Sending: %s\n", outgoing_message);
  radiometrix_send(outgoing_message, strlen(outgoing_message));
}


int main(int argc, char *argv[])
{

  /* Make sure we have the arguments we need */
  if (argc != 3) {
    fprintf(stderr, "Usage: %s <debug-server> <test-message>\n", argv[0]);
    exit(1);
  }
  outgoing_message = argv[2];

  radiometrix_set_debug_level(100);

  /* Initialize the radiometrix library */
  if (radiometrix_init_frag_debug(argv[1]) < 0) {
    fprintf(stderr, "radiometrix library failed - exiting\n");
    exit(1);
  }

  /* Now, loop forever: check for incoming packets, time out, send our
   * own packet.  First set up the signal handler. */
  signal(SIGALRM, send_packet);
  alarm(10);

  for (;;) {
    sleep(8);
  }

}
